/* Unicorn Emulator Engine */
/* By Nguyen Anh Quynh <aquynh@gmail.com>, 2015-2017 */
/* This file is released under LGPL2.
   See COPYING.LGPL2 in root directory for more details
*/

#ifndef QNICORN_X86_H
#define QNICORN_X86_H

#ifdef __cplusplus
extern "C" {
#endif

#include "platform.h"

//> X86 CPU
typedef enum qc_cpu_x86 {
    QC_CPU_X86_QEMU64 = 0,
    QC_CPU_X86_PHENOM,
    QC_CPU_X86_CORE2DUO,
    QC_CPU_X86_KVM64,
    QC_CPU_X86_QEMU32,
    QC_CPU_X86_KVM32,
    QC_CPU_X86_COREDUO,
    QC_CPU_X86_486,
    QC_CPU_X86_PENTIUM,
    QC_CPU_X86_PENTIUM2,
    QC_CPU_X86_PENTIUM3,
    QC_CPU_X86_ATHLON,
    QC_CPU_X86_N270,
    QC_CPU_X86_CONROE,
    QC_CPU_X86_PENRYN,
    QC_CPU_X86_NEHALEM,
    QC_CPU_X86_WESTMERE,
    QC_CPU_X86_SANDYBRIDGE,
    QC_CPU_X86_IVYBRIDGE,
    QC_CPU_X86_HASWELL,
    QC_CPU_X86_BROADWELL,
    QC_CPU_X86_SKYLAKE_CLIENT,
    QC_CPU_X86_SKYLAKE_SERVER,
    QC_CPU_X86_CASCADELAKE_SERVER,
    QC_CPU_X86_COOPERLAKE,
    QC_CPU_X86_ICELAKE_CLIENT,
    QC_CPU_X86_ICELAKE_SERVER,
    QC_CPU_X86_DENVERTON,
    QC_CPU_X86_SNOWRIDGE,
    QC_CPU_X86_KNIGHTSMILL,
    QC_CPU_X86_OPTERON_G1,
    QC_CPU_X86_OPTERON_G2,
    QC_CPU_X86_OPTERON_G3,
    QC_CPU_X86_OPTERON_G4,
    QC_CPU_X86_OPTERON_G5,
    QC_CPU_X86_EPYC,
    QC_CPU_X86_DHYANA,
    QC_CPU_X86_EPYC_ROME
} qc_cpu_x86;

// Memory-Management Register for instructions IDTR, GDTR, LDTR, TR.
// Borrow from SegmentCache in qemu/target-i386/cpu.h
typedef struct qc_x86_mmr {
    uint16_t selector; /* not used by GDTR and IDTR */
    uint64_t base;     /* handle 32 or 64 bit CPUs */
    uint32_t limit;
    uint32_t flags; /* not used by GDTR and IDTR */
} qc_x86_mmr;

// Model-Specific Register structure, use this with QC_X86_REG_MSR (as the
// register ID) in call to qc_reg_write/qc_reg_read() to manipulate MSRs.
typedef struct qc_x86_msr {
    uint32_t rid;   // MSR id
    uint64_t value; // MSR value
} qc_x86_msr;

// Callback function for tracing SYSCALL/SYSENTER (for qc_hook_intr())
// @user_data: user data passed to tracing APIs.
typedef void (*qc_cb_insn_syscall_t)(struct qc_struct *uc, void *user_data);

//> X86 registers
typedef enum qc_x86_reg {
    QC_X86_REG_INVALID = 0,
    QC_X86_REG_AH,
    QC_X86_REG_AL,
    QC_X86_REG_AX,
    QC_X86_REG_BH,
    QC_X86_REG_BL,
    QC_X86_REG_BP,
    QC_X86_REG_BPL,
    QC_X86_REG_BX,
    QC_X86_REG_CH,
    QC_X86_REG_CL,
    QC_X86_REG_CS,
    QC_X86_REG_CX,
    QC_X86_REG_DH,
    QC_X86_REG_DI,
    QC_X86_REG_DIL,
    QC_X86_REG_DL,
    QC_X86_REG_DS,
    QC_X86_REG_DX,
    QC_X86_REG_EAX,
    QC_X86_REG_EBP,
    QC_X86_REG_EBX,
    QC_X86_REG_ECX,
    QC_X86_REG_EDI,
    QC_X86_REG_EDX,
    QC_X86_REG_EFLAGS,
    QC_X86_REG_EIP,
    QC_X86_REG_ES,
    QC_X86_REG_ESI,
    QC_X86_REG_ESP,
    QC_X86_REG_FPSW,
    QC_X86_REG_FS,
    QC_X86_REG_GS,
    QC_X86_REG_IP,
    QC_X86_REG_RAX,
    QC_X86_REG_RBP,
    QC_X86_REG_RBX,
    QC_X86_REG_RCX,
    QC_X86_REG_RDI,
    QC_X86_REG_RDX,
    QC_X86_REG_RIP,
    QC_X86_REG_RSI,
    QC_X86_REG_RSP,
    QC_X86_REG_SI,
    QC_X86_REG_SIL,
    QC_X86_REG_SP,
    QC_X86_REG_SPL,
    QC_X86_REG_SS,
    QC_X86_REG_CR0,
    QC_X86_REG_CR1,
    QC_X86_REG_CR2,
    QC_X86_REG_CR3,
    QC_X86_REG_CR4,
    QC_X86_REG_CR8,
    QC_X86_REG_DR0,
    QC_X86_REG_DR1,
    QC_X86_REG_DR2,
    QC_X86_REG_DR3,
    QC_X86_REG_DR4,
    QC_X86_REG_DR5,
    QC_X86_REG_DR6,
    QC_X86_REG_DR7,
    QC_X86_REG_FP0,
    QC_X86_REG_FP1,
    QC_X86_REG_FP2,
    QC_X86_REG_FP3,
    QC_X86_REG_FP4,
    QC_X86_REG_FP5,
    QC_X86_REG_FP6,
    QC_X86_REG_FP7,
    QC_X86_REG_K0,
    QC_X86_REG_K1,
    QC_X86_REG_K2,
    QC_X86_REG_K3,
    QC_X86_REG_K4,
    QC_X86_REG_K5,
    QC_X86_REG_K6,
    QC_X86_REG_K7,
    QC_X86_REG_MM0,
    QC_X86_REG_MM1,
    QC_X86_REG_MM2,
    QC_X86_REG_MM3,
    QC_X86_REG_MM4,
    QC_X86_REG_MM5,
    QC_X86_REG_MM6,
    QC_X86_REG_MM7,
    QC_X86_REG_R8,
    QC_X86_REG_R9,
    QC_X86_REG_R10,
    QC_X86_REG_R11,
    QC_X86_REG_R12,
    QC_X86_REG_R13,
    QC_X86_REG_R14,
    QC_X86_REG_R15,
    QC_X86_REG_ST0,
    QC_X86_REG_ST1,
    QC_X86_REG_ST2,
    QC_X86_REG_ST3,
    QC_X86_REG_ST4,
    QC_X86_REG_ST5,
    QC_X86_REG_ST6,
    QC_X86_REG_ST7,
    QC_X86_REG_XMM0,
    QC_X86_REG_XMM1,
    QC_X86_REG_XMM2,
    QC_X86_REG_XMM3,
    QC_X86_REG_XMM4,
    QC_X86_REG_XMM5,
    QC_X86_REG_XMM6,
    QC_X86_REG_XMM7,
    QC_X86_REG_XMM8,
    QC_X86_REG_XMM9,
    QC_X86_REG_XMM10,
    QC_X86_REG_XMM11,
    QC_X86_REG_XMM12,
    QC_X86_REG_XMM13,
    QC_X86_REG_XMM14,
    QC_X86_REG_XMM15,
    QC_X86_REG_XMM16,
    QC_X86_REG_XMM17,
    QC_X86_REG_XMM18,
    QC_X86_REG_XMM19,
    QC_X86_REG_XMM20,
    QC_X86_REG_XMM21,
    QC_X86_REG_XMM22,
    QC_X86_REG_XMM23,
    QC_X86_REG_XMM24,
    QC_X86_REG_XMM25,
    QC_X86_REG_XMM26,
    QC_X86_REG_XMM27,
    QC_X86_REG_XMM28,
    QC_X86_REG_XMM29,
    QC_X86_REG_XMM30,
    QC_X86_REG_XMM31,
    QC_X86_REG_YMM0,
    QC_X86_REG_YMM1,
    QC_X86_REG_YMM2,
    QC_X86_REG_YMM3,
    QC_X86_REG_YMM4,
    QC_X86_REG_YMM5,
    QC_X86_REG_YMM6,
    QC_X86_REG_YMM7,
    QC_X86_REG_YMM8,
    QC_X86_REG_YMM9,
    QC_X86_REG_YMM10,
    QC_X86_REG_YMM11,
    QC_X86_REG_YMM12,
    QC_X86_REG_YMM13,
    QC_X86_REG_YMM14,
    QC_X86_REG_YMM15,
    QC_X86_REG_YMM16,
    QC_X86_REG_YMM17,
    QC_X86_REG_YMM18,
    QC_X86_REG_YMM19,
    QC_X86_REG_YMM20,
    QC_X86_REG_YMM21,
    QC_X86_REG_YMM22,
    QC_X86_REG_YMM23,
    QC_X86_REG_YMM24,
    QC_X86_REG_YMM25,
    QC_X86_REG_YMM26,
    QC_X86_REG_YMM27,
    QC_X86_REG_YMM28,
    QC_X86_REG_YMM29,
    QC_X86_REG_YMM30,
    QC_X86_REG_YMM31,
    QC_X86_REG_ZMM0,
    QC_X86_REG_ZMM1,
    QC_X86_REG_ZMM2,
    QC_X86_REG_ZMM3,
    QC_X86_REG_ZMM4,
    QC_X86_REG_ZMM5,
    QC_X86_REG_ZMM6,
    QC_X86_REG_ZMM7,
    QC_X86_REG_ZMM8,
    QC_X86_REG_ZMM9,
    QC_X86_REG_ZMM10,
    QC_X86_REG_ZMM11,
    QC_X86_REG_ZMM12,
    QC_X86_REG_ZMM13,
    QC_X86_REG_ZMM14,
    QC_X86_REG_ZMM15,
    QC_X86_REG_ZMM16,
    QC_X86_REG_ZMM17,
    QC_X86_REG_ZMM18,
    QC_X86_REG_ZMM19,
    QC_X86_REG_ZMM20,
    QC_X86_REG_ZMM21,
    QC_X86_REG_ZMM22,
    QC_X86_REG_ZMM23,
    QC_X86_REG_ZMM24,
    QC_X86_REG_ZMM25,
    QC_X86_REG_ZMM26,
    QC_X86_REG_ZMM27,
    QC_X86_REG_ZMM28,
    QC_X86_REG_ZMM29,
    QC_X86_REG_ZMM30,
    QC_X86_REG_ZMM31,
    QC_X86_REG_R8B,
    QC_X86_REG_R9B,
    QC_X86_REG_R10B,
    QC_X86_REG_R11B,
    QC_X86_REG_R12B,
    QC_X86_REG_R13B,
    QC_X86_REG_R14B,
    QC_X86_REG_R15B,
    QC_X86_REG_R8D,
    QC_X86_REG_R9D,
    QC_X86_REG_R10D,
    QC_X86_REG_R11D,
    QC_X86_REG_R12D,
    QC_X86_REG_R13D,
    QC_X86_REG_R14D,
    QC_X86_REG_R15D,
    QC_X86_REG_R8W,
    QC_X86_REG_R9W,
    QC_X86_REG_R10W,
    QC_X86_REG_R11W,
    QC_X86_REG_R12W,
    QC_X86_REG_R13W,
    QC_X86_REG_R14W,
    QC_X86_REG_R15W,
    QC_X86_REG_IDTR,
    QC_X86_REG_GDTR,
    QC_X86_REG_LDTR,
    QC_X86_REG_TR,
    QC_X86_REG_FPCW,
    QC_X86_REG_FPTAG,
    QC_X86_REG_MSR, // Model-Specific Register
    QC_X86_REG_MXCSR,
    QC_X86_REG_FS_BASE, // Base regs for x86_64
    QC_X86_REG_GS_BASE,
    QC_X86_REG_FLAGS,
    QC_X86_REG_RFLAGS,
    QC_X86_REG_ENDING // <-- mark the end of the list of registers
} qc_x86_reg;

//> X86 instructions
typedef enum qc_x86_insn {
    QC_X86_INS_INVALID = 0,

    QC_X86_INS_AAA,
    QC_X86_INS_AAD,
    QC_X86_INS_AAM,
    QC_X86_INS_AAS,
    QC_X86_INS_FABS,
    QC_X86_INS_ADC,
    QC_X86_INS_ADCX,
    QC_X86_INS_ADD,
    QC_X86_INS_ADDPD,
    QC_X86_INS_ADDPS,
    QC_X86_INS_ADDSD,
    QC_X86_INS_ADDSS,
    QC_X86_INS_ADDSUBPD,
    QC_X86_INS_ADDSUBPS,
    QC_X86_INS_FADD,
    QC_X86_INS_FIADD,
    QC_X86_INS_FADDP,
    QC_X86_INS_ADOX,
    QC_X86_INS_AESDECLAST,
    QC_X86_INS_AESDEC,
    QC_X86_INS_AESENCLAST,
    QC_X86_INS_AESENC,
    QC_X86_INS_AESIMC,
    QC_X86_INS_AESKEYGENASSIST,
    QC_X86_INS_AND,
    QC_X86_INS_ANDN,
    QC_X86_INS_ANDNPD,
    QC_X86_INS_ANDNPS,
    QC_X86_INS_ANDPD,
    QC_X86_INS_ANDPS,
    QC_X86_INS_ARPL,
    QC_X86_INS_BEXTR,
    QC_X86_INS_BLCFILL,
    QC_X86_INS_BLCI,
    QC_X86_INS_BLCIC,
    QC_X86_INS_BLCMSK,
    QC_X86_INS_BLCS,
    QC_X86_INS_BLENDPD,
    QC_X86_INS_BLENDPS,
    QC_X86_INS_BLENDVPD,
    QC_X86_INS_BLENDVPS,
    QC_X86_INS_BLSFILL,
    QC_X86_INS_BLSI,
    QC_X86_INS_BLSIC,
    QC_X86_INS_BLSMSK,
    QC_X86_INS_BLSR,
    QC_X86_INS_BOUND,
    QC_X86_INS_BSF,
    QC_X86_INS_BSR,
    QC_X86_INS_BSWAP,
    QC_X86_INS_BT,
    QC_X86_INS_BTC,
    QC_X86_INS_BTR,
    QC_X86_INS_BTS,
    QC_X86_INS_BZHI,
    QC_X86_INS_CALL,
    QC_X86_INS_CBW,
    QC_X86_INS_CDQ,
    QC_X86_INS_CDQE,
    QC_X86_INS_FCHS,
    QC_X86_INS_CLAC,
    QC_X86_INS_CLC,
    QC_X86_INS_CLD,
    QC_X86_INS_CLFLUSH,
    QC_X86_INS_CLFLUSHOPT,
    QC_X86_INS_CLGI,
    QC_X86_INS_CLI,
    QC_X86_INS_CLTS,
    QC_X86_INS_CLWB,
    QC_X86_INS_CMC,
    QC_X86_INS_CMOVA,
    QC_X86_INS_CMOVAE,
    QC_X86_INS_CMOVB,
    QC_X86_INS_CMOVBE,
    QC_X86_INS_FCMOVBE,
    QC_X86_INS_FCMOVB,
    QC_X86_INS_CMOVE,
    QC_X86_INS_FCMOVE,
    QC_X86_INS_CMOVG,
    QC_X86_INS_CMOVGE,
    QC_X86_INS_CMOVL,
    QC_X86_INS_CMOVLE,
    QC_X86_INS_FCMOVNBE,
    QC_X86_INS_FCMOVNB,
    QC_X86_INS_CMOVNE,
    QC_X86_INS_FCMOVNE,
    QC_X86_INS_CMOVNO,
    QC_X86_INS_CMOVNP,
    QC_X86_INS_FCMOVNU,
    QC_X86_INS_CMOVNS,
    QC_X86_INS_CMOVO,
    QC_X86_INS_CMOVP,
    QC_X86_INS_FCMOVU,
    QC_X86_INS_CMOVS,
    QC_X86_INS_CMP,
    QC_X86_INS_CMPPD,
    QC_X86_INS_CMPPS,
    QC_X86_INS_CMPSB,
    QC_X86_INS_CMPSD,
    QC_X86_INS_CMPSQ,
    QC_X86_INS_CMPSS,
    QC_X86_INS_CMPSW,
    QC_X86_INS_CMPXCHG16B,
    QC_X86_INS_CMPXCHG,
    QC_X86_INS_CMPXCHG8B,
    QC_X86_INS_COMISD,
    QC_X86_INS_COMISS,
    QC_X86_INS_FCOMP,
    QC_X86_INS_FCOMPI,
    QC_X86_INS_FCOMI,
    QC_X86_INS_FCOM,
    QC_X86_INS_FCOS,
    QC_X86_INS_CPUID,
    QC_X86_INS_CQO,
    QC_X86_INS_CRC32,
    QC_X86_INS_CVTDQ2PD,
    QC_X86_INS_CVTDQ2PS,
    QC_X86_INS_CVTPD2DQ,
    QC_X86_INS_CVTPD2PS,
    QC_X86_INS_CVTPS2DQ,
    QC_X86_INS_CVTPS2PD,
    QC_X86_INS_CVTSD2SI,
    QC_X86_INS_CVTSD2SS,
    QC_X86_INS_CVTSI2SD,
    QC_X86_INS_CVTSI2SS,
    QC_X86_INS_CVTSS2SD,
    QC_X86_INS_CVTSS2SI,
    QC_X86_INS_CVTTPD2DQ,
    QC_X86_INS_CVTTPS2DQ,
    QC_X86_INS_CVTTSD2SI,
    QC_X86_INS_CVTTSS2SI,
    QC_X86_INS_CWD,
    QC_X86_INS_CWDE,
    QC_X86_INS_DAA,
    QC_X86_INS_DAS,
    QC_X86_INS_DATA16,
    QC_X86_INS_DEC,
    QC_X86_INS_DIV,
    QC_X86_INS_DIVPD,
    QC_X86_INS_DIVPS,
    QC_X86_INS_FDIVR,
    QC_X86_INS_FIDIVR,
    QC_X86_INS_FDIVRP,
    QC_X86_INS_DIVSD,
    QC_X86_INS_DIVSS,
    QC_X86_INS_FDIV,
    QC_X86_INS_FIDIV,
    QC_X86_INS_FDIVP,
    QC_X86_INS_DPPD,
    QC_X86_INS_DPPS,
    QC_X86_INS_RET,
    QC_X86_INS_ENCLS,
    QC_X86_INS_ENCLU,
    QC_X86_INS_ENTER,
    QC_X86_INS_EXTRACTPS,
    QC_X86_INS_EXTRQ,
    QC_X86_INS_F2XM1,
    QC_X86_INS_LCALL,
    QC_X86_INS_LJMP,
    QC_X86_INS_FBLD,
    QC_X86_INS_FBSTP,
    QC_X86_INS_FCOMPP,
    QC_X86_INS_FDECSTP,
    QC_X86_INS_FEMMS,
    QC_X86_INS_FFREE,
    QC_X86_INS_FICOM,
    QC_X86_INS_FICOMP,
    QC_X86_INS_FINCSTP,
    QC_X86_INS_FLDCW,
    QC_X86_INS_FLDENV,
    QC_X86_INS_FLDL2E,
    QC_X86_INS_FLDL2T,
    QC_X86_INS_FLDLG2,
    QC_X86_INS_FLDLN2,
    QC_X86_INS_FLDPI,
    QC_X86_INS_FNCLEX,
    QC_X86_INS_FNINIT,
    QC_X86_INS_FNOP,
    QC_X86_INS_FNSTCW,
    QC_X86_INS_FNSTSW,
    QC_X86_INS_FPATAN,
    QC_X86_INS_FPREM,
    QC_X86_INS_FPREM1,
    QC_X86_INS_FPTAN,
    QC_X86_INS_FFREEP,
    QC_X86_INS_FRNDINT,
    QC_X86_INS_FRSTOR,
    QC_X86_INS_FNSAVE,
    QC_X86_INS_FSCALE,
    QC_X86_INS_FSETPM,
    QC_X86_INS_FSINCOS,
    QC_X86_INS_FNSTENV,
    QC_X86_INS_FXAM,
    QC_X86_INS_FXRSTOR,
    QC_X86_INS_FXRSTOR64,
    QC_X86_INS_FXSAVE,
    QC_X86_INS_FXSAVE64,
    QC_X86_INS_FXTRACT,
    QC_X86_INS_FYL2X,
    QC_X86_INS_FYL2XP1,
    QC_X86_INS_MOVAPD,
    QC_X86_INS_MOVAPS,
    QC_X86_INS_ORPD,
    QC_X86_INS_ORPS,
    QC_X86_INS_VMOVAPD,
    QC_X86_INS_VMOVAPS,
    QC_X86_INS_XORPD,
    QC_X86_INS_XORPS,
    QC_X86_INS_GETSEC,
    QC_X86_INS_HADDPD,
    QC_X86_INS_HADDPS,
    QC_X86_INS_HLT,
    QC_X86_INS_HSUBPD,
    QC_X86_INS_HSUBPS,
    QC_X86_INS_IDIV,
    QC_X86_INS_FILD,
    QC_X86_INS_IMUL,
    QC_X86_INS_IN,
    QC_X86_INS_INC,
    QC_X86_INS_INSB,
    QC_X86_INS_INSERTPS,
    QC_X86_INS_INSERTQ,
    QC_X86_INS_INSD,
    QC_X86_INS_INSW,
    QC_X86_INS_INT,
    QC_X86_INS_INT1,
    QC_X86_INS_INT3,
    QC_X86_INS_INTO,
    QC_X86_INS_INVD,
    QC_X86_INS_INVEPT,
    QC_X86_INS_INVLPG,
    QC_X86_INS_INVLPGA,
    QC_X86_INS_INVPCID,
    QC_X86_INS_INVVPID,
    QC_X86_INS_IRET,
    QC_X86_INS_IRETD,
    QC_X86_INS_IRETQ,
    QC_X86_INS_FISTTP,
    QC_X86_INS_FIST,
    QC_X86_INS_FISTP,
    QC_X86_INS_UCOMISD,
    QC_X86_INS_UCOMISS,
    QC_X86_INS_VCOMISD,
    QC_X86_INS_VCOMISS,
    QC_X86_INS_VCVTSD2SS,
    QC_X86_INS_VCVTSI2SD,
    QC_X86_INS_VCVTSI2SS,
    QC_X86_INS_VCVTSS2SD,
    QC_X86_INS_VCVTTSD2SI,
    QC_X86_INS_VCVTTSD2USI,
    QC_X86_INS_VCVTTSS2SI,
    QC_X86_INS_VCVTTSS2USI,
    QC_X86_INS_VCVTUSI2SD,
    QC_X86_INS_VCVTUSI2SS,
    QC_X86_INS_VUCOMISD,
    QC_X86_INS_VUCOMISS,
    QC_X86_INS_JAE,
    QC_X86_INS_JA,
    QC_X86_INS_JBE,
    QC_X86_INS_JB,
    QC_X86_INS_JCXZ,
    QC_X86_INS_JECXZ,
    QC_X86_INS_JE,
    QC_X86_INS_JGE,
    QC_X86_INS_JG,
    QC_X86_INS_JLE,
    QC_X86_INS_JL,
    QC_X86_INS_JMP,
    QC_X86_INS_JNE,
    QC_X86_INS_JNO,
    QC_X86_INS_JNP,
    QC_X86_INS_JNS,
    QC_X86_INS_JO,
    QC_X86_INS_JP,
    QC_X86_INS_JRCXZ,
    QC_X86_INS_JS,
    QC_X86_INS_KANDB,
    QC_X86_INS_KANDD,
    QC_X86_INS_KANDNB,
    QC_X86_INS_KANDND,
    QC_X86_INS_KANDNQ,
    QC_X86_INS_KANDNW,
    QC_X86_INS_KANDQ,
    QC_X86_INS_KANDW,
    QC_X86_INS_KMOVB,
    QC_X86_INS_KMOVD,
    QC_X86_INS_KMOVQ,
    QC_X86_INS_KMOVW,
    QC_X86_INS_KNOTB,
    QC_X86_INS_KNOTD,
    QC_X86_INS_KNOTQ,
    QC_X86_INS_KNOTW,
    QC_X86_INS_KORB,
    QC_X86_INS_KORD,
    QC_X86_INS_KORQ,
    QC_X86_INS_KORTESTB,
    QC_X86_INS_KORTESTD,
    QC_X86_INS_KORTESTQ,
    QC_X86_INS_KORTESTW,
    QC_X86_INS_KORW,
    QC_X86_INS_KSHIFTLB,
    QC_X86_INS_KSHIFTLD,
    QC_X86_INS_KSHIFTLQ,
    QC_X86_INS_KSHIFTLW,
    QC_X86_INS_KSHIFTRB,
    QC_X86_INS_KSHIFTRD,
    QC_X86_INS_KSHIFTRQ,
    QC_X86_INS_KSHIFTRW,
    QC_X86_INS_KUNPCKBW,
    QC_X86_INS_KXNORB,
    QC_X86_INS_KXNORD,
    QC_X86_INS_KXNORQ,
    QC_X86_INS_KXNORW,
    QC_X86_INS_KXORB,
    QC_X86_INS_KXORD,
    QC_X86_INS_KXORQ,
    QC_X86_INS_KXORW,
    QC_X86_INS_LAHF,
    QC_X86_INS_LAR,
    QC_X86_INS_LDDQU,
    QC_X86_INS_LDMXCSR,
    QC_X86_INS_LDS,
    QC_X86_INS_FLDZ,
    QC_X86_INS_FLD1,
    QC_X86_INS_FLD,
    QC_X86_INS_LEA,
    QC_X86_INS_LEAVE,
    QC_X86_INS_LES,
    QC_X86_INS_LFENCE,
    QC_X86_INS_LFS,
    QC_X86_INS_LGDT,
    QC_X86_INS_LGS,
    QC_X86_INS_LIDT,
    QC_X86_INS_LLDT,
    QC_X86_INS_LMSW,
    QC_X86_INS_OR,
    QC_X86_INS_SUB,
    QC_X86_INS_XOR,
    QC_X86_INS_LODSB,
    QC_X86_INS_LODSD,
    QC_X86_INS_LODSQ,
    QC_X86_INS_LODSW,
    QC_X86_INS_LOOP,
    QC_X86_INS_LOOPE,
    QC_X86_INS_LOOPNE,
    QC_X86_INS_RETF,
    QC_X86_INS_RETFQ,
    QC_X86_INS_LSL,
    QC_X86_INS_LSS,
    QC_X86_INS_LTR,
    QC_X86_INS_XADD,
    QC_X86_INS_LZCNT,
    QC_X86_INS_MASKMOVDQU,
    QC_X86_INS_MAXPD,
    QC_X86_INS_MAXPS,
    QC_X86_INS_MAXSD,
    QC_X86_INS_MAXSS,
    QC_X86_INS_MFENCE,
    QC_X86_INS_MINPD,
    QC_X86_INS_MINPS,
    QC_X86_INS_MINSD,
    QC_X86_INS_MINSS,
    QC_X86_INS_CVTPD2PI,
    QC_X86_INS_CVTPI2PD,
    QC_X86_INS_CVTPI2PS,
    QC_X86_INS_CVTPS2PI,
    QC_X86_INS_CVTTPD2PI,
    QC_X86_INS_CVTTPS2PI,
    QC_X86_INS_EMMS,
    QC_X86_INS_MASKMOVQ,
    QC_X86_INS_MOVD,
    QC_X86_INS_MOVDQ2Q,
    QC_X86_INS_MOVNTQ,
    QC_X86_INS_MOVQ2DQ,
    QC_X86_INS_MOVQ,
    QC_X86_INS_PABSB,
    QC_X86_INS_PABSD,
    QC_X86_INS_PABSW,
    QC_X86_INS_PACKSSDW,
    QC_X86_INS_PACKSSWB,
    QC_X86_INS_PACKUSWB,
    QC_X86_INS_PADDB,
    QC_X86_INS_PADDD,
    QC_X86_INS_PADDQ,
    QC_X86_INS_PADDSB,
    QC_X86_INS_PADDSW,
    QC_X86_INS_PADDUSB,
    QC_X86_INS_PADDUSW,
    QC_X86_INS_PADDW,
    QC_X86_INS_PALIGNR,
    QC_X86_INS_PANDN,
    QC_X86_INS_PAND,
    QC_X86_INS_PAVGB,
    QC_X86_INS_PAVGW,
    QC_X86_INS_PCMPEQB,
    QC_X86_INS_PCMPEQD,
    QC_X86_INS_PCMPEQW,
    QC_X86_INS_PCMPGTB,
    QC_X86_INS_PCMPGTD,
    QC_X86_INS_PCMPGTW,
    QC_X86_INS_PEXTRW,
    QC_X86_INS_PHADDSW,
    QC_X86_INS_PHADDW,
    QC_X86_INS_PHADDD,
    QC_X86_INS_PHSUBD,
    QC_X86_INS_PHSUBSW,
    QC_X86_INS_PHSUBW,
    QC_X86_INS_PINSRW,
    QC_X86_INS_PMADDUBSW,
    QC_X86_INS_PMADDWD,
    QC_X86_INS_PMAXSW,
    QC_X86_INS_PMAXUB,
    QC_X86_INS_PMINSW,
    QC_X86_INS_PMINUB,
    QC_X86_INS_PMOVMSKB,
    QC_X86_INS_PMULHRSW,
    QC_X86_INS_PMULHUW,
    QC_X86_INS_PMULHW,
    QC_X86_INS_PMULLW,
    QC_X86_INS_PMULUDQ,
    QC_X86_INS_POR,
    QC_X86_INS_PSADBW,
    QC_X86_INS_PSHUFB,
    QC_X86_INS_PSHUFW,
    QC_X86_INS_PSIGNB,
    QC_X86_INS_PSIGND,
    QC_X86_INS_PSIGNW,
    QC_X86_INS_PSLLD,
    QC_X86_INS_PSLLQ,
    QC_X86_INS_PSLLW,
    QC_X86_INS_PSRAD,
    QC_X86_INS_PSRAW,
    QC_X86_INS_PSRLD,
    QC_X86_INS_PSRLQ,
    QC_X86_INS_PSRLW,
    QC_X86_INS_PSUBB,
    QC_X86_INS_PSUBD,
    QC_X86_INS_PSUBQ,
    QC_X86_INS_PSUBSB,
    QC_X86_INS_PSUBSW,
    QC_X86_INS_PSUBUSB,
    QC_X86_INS_PSUBUSW,
    QC_X86_INS_PSUBW,
    QC_X86_INS_PUNPCKHBW,
    QC_X86_INS_PUNPCKHDQ,
    QC_X86_INS_PUNPCKHWD,
    QC_X86_INS_PUNPCKLBW,
    QC_X86_INS_PUNPCKLDQ,
    QC_X86_INS_PUNPCKLWD,
    QC_X86_INS_PXOR,
    QC_X86_INS_MONITOR,
    QC_X86_INS_MONTMUL,
    QC_X86_INS_MOV,
    QC_X86_INS_MOVABS,
    QC_X86_INS_MOVBE,
    QC_X86_INS_MOVDDUP,
    QC_X86_INS_MOVDQA,
    QC_X86_INS_MOVDQU,
    QC_X86_INS_MOVHLPS,
    QC_X86_INS_MOVHPD,
    QC_X86_INS_MOVHPS,
    QC_X86_INS_MOVLHPS,
    QC_X86_INS_MOVLPD,
    QC_X86_INS_MOVLPS,
    QC_X86_INS_MOVMSKPD,
    QC_X86_INS_MOVMSKPS,
    QC_X86_INS_MOVNTDQA,
    QC_X86_INS_MOVNTDQ,
    QC_X86_INS_MOVNTI,
    QC_X86_INS_MOVNTPD,
    QC_X86_INS_MOVNTPS,
    QC_X86_INS_MOVNTSD,
    QC_X86_INS_MOVNTSS,
    QC_X86_INS_MOVSB,
    QC_X86_INS_MOVSD,
    QC_X86_INS_MOVSHDUP,
    QC_X86_INS_MOVSLDUP,
    QC_X86_INS_MOVSQ,
    QC_X86_INS_MOVSS,
    QC_X86_INS_MOVSW,
    QC_X86_INS_MOVSX,
    QC_X86_INS_MOVSXD,
    QC_X86_INS_MOVUPD,
    QC_X86_INS_MOVUPS,
    QC_X86_INS_MOVZX,
    QC_X86_INS_MPSADBW,
    QC_X86_INS_MUL,
    QC_X86_INS_MULPD,
    QC_X86_INS_MULPS,
    QC_X86_INS_MULSD,
    QC_X86_INS_MULSS,
    QC_X86_INS_MULX,
    QC_X86_INS_FMUL,
    QC_X86_INS_FIMUL,
    QC_X86_INS_FMULP,
    QC_X86_INS_MWAIT,
    QC_X86_INS_NEG,
    QC_X86_INS_NOP,
    QC_X86_INS_NOT,
    QC_X86_INS_OUT,
    QC_X86_INS_OUTSB,
    QC_X86_INS_OUTSD,
    QC_X86_INS_OUTSW,
    QC_X86_INS_PACKUSDW,
    QC_X86_INS_PAUSE,
    QC_X86_INS_PAVGUSB,
    QC_X86_INS_PBLENDVB,
    QC_X86_INS_PBLENDW,
    QC_X86_INS_PCLMULQDQ,
    QC_X86_INS_PCMPEQQ,
    QC_X86_INS_PCMPESTRI,
    QC_X86_INS_PCMPESTRM,
    QC_X86_INS_PCMPGTQ,
    QC_X86_INS_PCMPISTRI,
    QC_X86_INS_PCMPISTRM,
    QC_X86_INS_PCOMMIT,
    QC_X86_INS_PDEP,
    QC_X86_INS_PEXT,
    QC_X86_INS_PEXTRB,
    QC_X86_INS_PEXTRD,
    QC_X86_INS_PEXTRQ,
    QC_X86_INS_PF2ID,
    QC_X86_INS_PF2IW,
    QC_X86_INS_PFACC,
    QC_X86_INS_PFADD,
    QC_X86_INS_PFCMPEQ,
    QC_X86_INS_PFCMPGE,
    QC_X86_INS_PFCMPGT,
    QC_X86_INS_PFMAX,
    QC_X86_INS_PFMIN,
    QC_X86_INS_PFMUL,
    QC_X86_INS_PFNACC,
    QC_X86_INS_PFPNACC,
    QC_X86_INS_PFRCPIT1,
    QC_X86_INS_PFRCPIT2,
    QC_X86_INS_PFRCP,
    QC_X86_INS_PFRSQIT1,
    QC_X86_INS_PFRSQRT,
    QC_X86_INS_PFSUBR,
    QC_X86_INS_PFSUB,
    QC_X86_INS_PHMINPOSUW,
    QC_X86_INS_PI2FD,
    QC_X86_INS_PI2FW,
    QC_X86_INS_PINSRB,
    QC_X86_INS_PINSRD,
    QC_X86_INS_PINSRQ,
    QC_X86_INS_PMAXSB,
    QC_X86_INS_PMAXSD,
    QC_X86_INS_PMAXUD,
    QC_X86_INS_PMAXUW,
    QC_X86_INS_PMINSB,
    QC_X86_INS_PMINSD,
    QC_X86_INS_PMINUD,
    QC_X86_INS_PMINUW,
    QC_X86_INS_PMOVSXBD,
    QC_X86_INS_PMOVSXBQ,
    QC_X86_INS_PMOVSXBW,
    QC_X86_INS_PMOVSXDQ,
    QC_X86_INS_PMOVSXWD,
    QC_X86_INS_PMOVSXWQ,
    QC_X86_INS_PMOVZXBD,
    QC_X86_INS_PMOVZXBQ,
    QC_X86_INS_PMOVZXBW,
    QC_X86_INS_PMOVZXDQ,
    QC_X86_INS_PMOVZXWD,
    QC_X86_INS_PMOVZXWQ,
    QC_X86_INS_PMULDQ,
    QC_X86_INS_PMULHRW,
    QC_X86_INS_PMULLD,
    QC_X86_INS_POP,
    QC_X86_INS_POPAW,
    QC_X86_INS_POPAL,
    QC_X86_INS_POPCNT,
    QC_X86_INS_POPF,
    QC_X86_INS_POPFD,
    QC_X86_INS_POPFQ,
    QC_X86_INS_PREFETCH,
    QC_X86_INS_PREFETCHNTA,
    QC_X86_INS_PREFETCHT0,
    QC_X86_INS_PREFETCHT1,
    QC_X86_INS_PREFETCHT2,
    QC_X86_INS_PREFETCHW,
    QC_X86_INS_PSHUFD,
    QC_X86_INS_PSHUFHW,
    QC_X86_INS_PSHUFLW,
    QC_X86_INS_PSLLDQ,
    QC_X86_INS_PSRLDQ,
    QC_X86_INS_PSWAPD,
    QC_X86_INS_PTEST,
    QC_X86_INS_PUNPCKHQDQ,
    QC_X86_INS_PUNPCKLQDQ,
    QC_X86_INS_PUSH,
    QC_X86_INS_PUSHAW,
    QC_X86_INS_PUSHAL,
    QC_X86_INS_PUSHF,
    QC_X86_INS_PUSHFD,
    QC_X86_INS_PUSHFQ,
    QC_X86_INS_RCL,
    QC_X86_INS_RCPPS,
    QC_X86_INS_RCPSS,
    QC_X86_INS_RCR,
    QC_X86_INS_RDFSBASE,
    QC_X86_INS_RDGSBASE,
    QC_X86_INS_RDMSR,
    QC_X86_INS_RDPMC,
    QC_X86_INS_RDRAND,
    QC_X86_INS_RDSEED,
    QC_X86_INS_RDTSC,
    QC_X86_INS_RDTSCP,
    QC_X86_INS_ROL,
    QC_X86_INS_ROR,
    QC_X86_INS_RORX,
    QC_X86_INS_ROUNDPD,
    QC_X86_INS_ROUNDPS,
    QC_X86_INS_ROUNDSD,
    QC_X86_INS_ROUNDSS,
    QC_X86_INS_RSM,
    QC_X86_INS_RSQRTPS,
    QC_X86_INS_RSQRTSS,
    QC_X86_INS_SAHF,
    QC_X86_INS_SAL,
    QC_X86_INS_SALC,
    QC_X86_INS_SAR,
    QC_X86_INS_SARX,
    QC_X86_INS_SBB,
    QC_X86_INS_SCASB,
    QC_X86_INS_SCASD,
    QC_X86_INS_SCASQ,
    QC_X86_INS_SCASW,
    QC_X86_INS_SETAE,
    QC_X86_INS_SETA,
    QC_X86_INS_SETBE,
    QC_X86_INS_SETB,
    QC_X86_INS_SETE,
    QC_X86_INS_SETGE,
    QC_X86_INS_SETG,
    QC_X86_INS_SETLE,
    QC_X86_INS_SETL,
    QC_X86_INS_SETNE,
    QC_X86_INS_SETNO,
    QC_X86_INS_SETNP,
    QC_X86_INS_SETNS,
    QC_X86_INS_SETO,
    QC_X86_INS_SETP,
    QC_X86_INS_SETS,
    QC_X86_INS_SFENCE,
    QC_X86_INS_SGDT,
    QC_X86_INS_SHA1MSG1,
    QC_X86_INS_SHA1MSG2,
    QC_X86_INS_SHA1NEXTE,
    QC_X86_INS_SHA1RNDS4,
    QC_X86_INS_SHA256MSG1,
    QC_X86_INS_SHA256MSG2,
    QC_X86_INS_SHA256RNDS2,
    QC_X86_INS_SHL,
    QC_X86_INS_SHLD,
    QC_X86_INS_SHLX,
    QC_X86_INS_SHR,
    QC_X86_INS_SHRD,
    QC_X86_INS_SHRX,
    QC_X86_INS_SHUFPD,
    QC_X86_INS_SHUFPS,
    QC_X86_INS_SIDT,
    QC_X86_INS_FSIN,
    QC_X86_INS_SKINIT,
    QC_X86_INS_SLDT,
    QC_X86_INS_SMSW,
    QC_X86_INS_SQRTPD,
    QC_X86_INS_SQRTPS,
    QC_X86_INS_SQRTSD,
    QC_X86_INS_SQRTSS,
    QC_X86_INS_FSQRT,
    QC_X86_INS_STAC,
    QC_X86_INS_STC,
    QC_X86_INS_STD,
    QC_X86_INS_STGI,
    QC_X86_INS_STI,
    QC_X86_INS_STMXCSR,
    QC_X86_INS_STOSB,
    QC_X86_INS_STOSD,
    QC_X86_INS_STOSQ,
    QC_X86_INS_STOSW,
    QC_X86_INS_STR,
    QC_X86_INS_FST,
    QC_X86_INS_FSTP,
    QC_X86_INS_FSTPNCE,
    QC_X86_INS_FXCH,
    QC_X86_INS_SUBPD,
    QC_X86_INS_SUBPS,
    QC_X86_INS_FSUBR,
    QC_X86_INS_FISUBR,
    QC_X86_INS_FSUBRP,
    QC_X86_INS_SUBSD,
    QC_X86_INS_SUBSS,
    QC_X86_INS_FSUB,
    QC_X86_INS_FISUB,
    QC_X86_INS_FSUBP,
    QC_X86_INS_SWAPGS,
    QC_X86_INS_SYSCALL,
    QC_X86_INS_SYSENTER,
    QC_X86_INS_SYSEXIT,
    QC_X86_INS_SYSRET,
    QC_X86_INS_T1MSKC,
    QC_X86_INS_TEST,
    QC_X86_INS_UD2,
    QC_X86_INS_FTST,
    QC_X86_INS_TZCNT,
    QC_X86_INS_TZMSK,
    QC_X86_INS_FUCOMPI,
    QC_X86_INS_FUCOMI,
    QC_X86_INS_FUCOMPP,
    QC_X86_INS_FUCOMP,
    QC_X86_INS_FUCOM,
    QC_X86_INS_UD2B,
    QC_X86_INS_UNPCKHPD,
    QC_X86_INS_UNPCKHPS,
    QC_X86_INS_UNPCKLPD,
    QC_X86_INS_UNPCKLPS,
    QC_X86_INS_VADDPD,
    QC_X86_INS_VADDPS,
    QC_X86_INS_VADDSD,
    QC_X86_INS_VADDSS,
    QC_X86_INS_VADDSUBPD,
    QC_X86_INS_VADDSUBPS,
    QC_X86_INS_VAESDECLAST,
    QC_X86_INS_VAESDEC,
    QC_X86_INS_VAESENCLAST,
    QC_X86_INS_VAESENC,
    QC_X86_INS_VAESIMC,
    QC_X86_INS_VAESKEYGENASSIST,
    QC_X86_INS_VALIGND,
    QC_X86_INS_VALIGNQ,
    QC_X86_INS_VANDNPD,
    QC_X86_INS_VANDNPS,
    QC_X86_INS_VANDPD,
    QC_X86_INS_VANDPS,
    QC_X86_INS_VBLENDMPD,
    QC_X86_INS_VBLENDMPS,
    QC_X86_INS_VBLENDPD,
    QC_X86_INS_VBLENDPS,
    QC_X86_INS_VBLENDVPD,
    QC_X86_INS_VBLENDVPS,
    QC_X86_INS_VBROADCASTF128,
    QC_X86_INS_VBROADCASTI32X4,
    QC_X86_INS_VBROADCASTI64X4,
    QC_X86_INS_VBROADCASTSD,
    QC_X86_INS_VBROADCASTSS,
    QC_X86_INS_VCMPPD,
    QC_X86_INS_VCMPPS,
    QC_X86_INS_VCMPSD,
    QC_X86_INS_VCMPSS,
    QC_X86_INS_VCOMPRESSPD,
    QC_X86_INS_VCOMPRESSPS,
    QC_X86_INS_VCVTDQ2PD,
    QC_X86_INS_VCVTDQ2PS,
    QC_X86_INS_VCVTPD2DQX,
    QC_X86_INS_VCVTPD2DQ,
    QC_X86_INS_VCVTPD2PSX,
    QC_X86_INS_VCVTPD2PS,
    QC_X86_INS_VCVTPD2UDQ,
    QC_X86_INS_VCVTPH2PS,
    QC_X86_INS_VCVTPS2DQ,
    QC_X86_INS_VCVTPS2PD,
    QC_X86_INS_VCVTPS2PH,
    QC_X86_INS_VCVTPS2UDQ,
    QC_X86_INS_VCVTSD2SI,
    QC_X86_INS_VCVTSD2USI,
    QC_X86_INS_VCVTSS2SI,
    QC_X86_INS_VCVTSS2USI,
    QC_X86_INS_VCVTTPD2DQX,
    QC_X86_INS_VCVTTPD2DQ,
    QC_X86_INS_VCVTTPD2UDQ,
    QC_X86_INS_VCVTTPS2DQ,
    QC_X86_INS_VCVTTPS2UDQ,
    QC_X86_INS_VCVTUDQ2PD,
    QC_X86_INS_VCVTUDQ2PS,
    QC_X86_INS_VDIVPD,
    QC_X86_INS_VDIVPS,
    QC_X86_INS_VDIVSD,
    QC_X86_INS_VDIVSS,
    QC_X86_INS_VDPPD,
    QC_X86_INS_VDPPS,
    QC_X86_INS_VERR,
    QC_X86_INS_VERW,
    QC_X86_INS_VEXP2PD,
    QC_X86_INS_VEXP2PS,
    QC_X86_INS_VEXPANDPD,
    QC_X86_INS_VEXPANDPS,
    QC_X86_INS_VEXTRACTF128,
    QC_X86_INS_VEXTRACTF32X4,
    QC_X86_INS_VEXTRACTF64X4,
    QC_X86_INS_VEXTRACTI128,
    QC_X86_INS_VEXTRACTI32X4,
    QC_X86_INS_VEXTRACTI64X4,
    QC_X86_INS_VEXTRACTPS,
    QC_X86_INS_VFMADD132PD,
    QC_X86_INS_VFMADD132PS,
    QC_X86_INS_VFMADDPD,
    QC_X86_INS_VFMADD213PD,
    QC_X86_INS_VFMADD231PD,
    QC_X86_INS_VFMADDPS,
    QC_X86_INS_VFMADD213PS,
    QC_X86_INS_VFMADD231PS,
    QC_X86_INS_VFMADDSD,
    QC_X86_INS_VFMADD213SD,
    QC_X86_INS_VFMADD132SD,
    QC_X86_INS_VFMADD231SD,
    QC_X86_INS_VFMADDSS,
    QC_X86_INS_VFMADD213SS,
    QC_X86_INS_VFMADD132SS,
    QC_X86_INS_VFMADD231SS,
    QC_X86_INS_VFMADDSUB132PD,
    QC_X86_INS_VFMADDSUB132PS,
    QC_X86_INS_VFMADDSUBPD,
    QC_X86_INS_VFMADDSUB213PD,
    QC_X86_INS_VFMADDSUB231PD,
    QC_X86_INS_VFMADDSUBPS,
    QC_X86_INS_VFMADDSUB213PS,
    QC_X86_INS_VFMADDSUB231PS,
    QC_X86_INS_VFMSUB132PD,
    QC_X86_INS_VFMSUB132PS,
    QC_X86_INS_VFMSUBADD132PD,
    QC_X86_INS_VFMSUBADD132PS,
    QC_X86_INS_VFMSUBADDPD,
    QC_X86_INS_VFMSUBADD213PD,
    QC_X86_INS_VFMSUBADD231PD,
    QC_X86_INS_VFMSUBADDPS,
    QC_X86_INS_VFMSUBADD213PS,
    QC_X86_INS_VFMSUBADD231PS,
    QC_X86_INS_VFMSUBPD,
    QC_X86_INS_VFMSUB213PD,
    QC_X86_INS_VFMSUB231PD,
    QC_X86_INS_VFMSUBPS,
    QC_X86_INS_VFMSUB213PS,
    QC_X86_INS_VFMSUB231PS,
    QC_X86_INS_VFMSUBSD,
    QC_X86_INS_VFMSUB213SD,
    QC_X86_INS_VFMSUB132SD,
    QC_X86_INS_VFMSUB231SD,
    QC_X86_INS_VFMSUBSS,
    QC_X86_INS_VFMSUB213SS,
    QC_X86_INS_VFMSUB132SS,
    QC_X86_INS_VFMSUB231SS,
    QC_X86_INS_VFNMADD132PD,
    QC_X86_INS_VFNMADD132PS,
    QC_X86_INS_VFNMADDPD,
    QC_X86_INS_VFNMADD213PD,
    QC_X86_INS_VFNMADD231PD,
    QC_X86_INS_VFNMADDPS,
    QC_X86_INS_VFNMADD213PS,
    QC_X86_INS_VFNMADD231PS,
    QC_X86_INS_VFNMADDSD,
    QC_X86_INS_VFNMADD213SD,
    QC_X86_INS_VFNMADD132SD,
    QC_X86_INS_VFNMADD231SD,
    QC_X86_INS_VFNMADDSS,
    QC_X86_INS_VFNMADD213SS,
    QC_X86_INS_VFNMADD132SS,
    QC_X86_INS_VFNMADD231SS,
    QC_X86_INS_VFNMSUB132PD,
    QC_X86_INS_VFNMSUB132PS,
    QC_X86_INS_VFNMSUBPD,
    QC_X86_INS_VFNMSUB213PD,
    QC_X86_INS_VFNMSUB231PD,
    QC_X86_INS_VFNMSUBPS,
    QC_X86_INS_VFNMSUB213PS,
    QC_X86_INS_VFNMSUB231PS,
    QC_X86_INS_VFNMSUBSD,
    QC_X86_INS_VFNMSUB213SD,
    QC_X86_INS_VFNMSUB132SD,
    QC_X86_INS_VFNMSUB231SD,
    QC_X86_INS_VFNMSUBSS,
    QC_X86_INS_VFNMSUB213SS,
    QC_X86_INS_VFNMSUB132SS,
    QC_X86_INS_VFNMSUB231SS,
    QC_X86_INS_VFRCZPD,
    QC_X86_INS_VFRCZPS,
    QC_X86_INS_VFRCZSD,
    QC_X86_INS_VFRCZSS,
    QC_X86_INS_VORPD,
    QC_X86_INS_VORPS,
    QC_X86_INS_VXORPD,
    QC_X86_INS_VXORPS,
    QC_X86_INS_VGATHERDPD,
    QC_X86_INS_VGATHERDPS,
    QC_X86_INS_VGATHERPF0DPD,
    QC_X86_INS_VGATHERPF0DPS,
    QC_X86_INS_VGATHERPF0QPD,
    QC_X86_INS_VGATHERPF0QPS,
    QC_X86_INS_VGATHERPF1DPD,
    QC_X86_INS_VGATHERPF1DPS,
    QC_X86_INS_VGATHERPF1QPD,
    QC_X86_INS_VGATHERPF1QPS,
    QC_X86_INS_VGATHERQPD,
    QC_X86_INS_VGATHERQPS,
    QC_X86_INS_VHADDPD,
    QC_X86_INS_VHADDPS,
    QC_X86_INS_VHSUBPD,
    QC_X86_INS_VHSUBPS,
    QC_X86_INS_VINSERTF128,
    QC_X86_INS_VINSERTF32X4,
    QC_X86_INS_VINSERTF32X8,
    QC_X86_INS_VINSERTF64X2,
    QC_X86_INS_VINSERTF64X4,
    QC_X86_INS_VINSERTI128,
    QC_X86_INS_VINSERTI32X4,
    QC_X86_INS_VINSERTI32X8,
    QC_X86_INS_VINSERTI64X2,
    QC_X86_INS_VINSERTI64X4,
    QC_X86_INS_VINSERTPS,
    QC_X86_INS_VLDDQU,
    QC_X86_INS_VLDMXCSR,
    QC_X86_INS_VMASKMOVDQU,
    QC_X86_INS_VMASKMOVPD,
    QC_X86_INS_VMASKMOVPS,
    QC_X86_INS_VMAXPD,
    QC_X86_INS_VMAXPS,
    QC_X86_INS_VMAXSD,
    QC_X86_INS_VMAXSS,
    QC_X86_INS_VMCALL,
    QC_X86_INS_VMCLEAR,
    QC_X86_INS_VMFUNC,
    QC_X86_INS_VMINPD,
    QC_X86_INS_VMINPS,
    QC_X86_INS_VMINSD,
    QC_X86_INS_VMINSS,
    QC_X86_INS_VMLAUNCH,
    QC_X86_INS_VMLOAD,
    QC_X86_INS_VMMCALL,
    QC_X86_INS_VMOVQ,
    QC_X86_INS_VMOVDDUP,
    QC_X86_INS_VMOVD,
    QC_X86_INS_VMOVDQA32,
    QC_X86_INS_VMOVDQA64,
    QC_X86_INS_VMOVDQA,
    QC_X86_INS_VMOVDQU16,
    QC_X86_INS_VMOVDQU32,
    QC_X86_INS_VMOVDQU64,
    QC_X86_INS_VMOVDQU8,
    QC_X86_INS_VMOVDQU,
    QC_X86_INS_VMOVHLPS,
    QC_X86_INS_VMOVHPD,
    QC_X86_INS_VMOVHPS,
    QC_X86_INS_VMOVLHPS,
    QC_X86_INS_VMOVLPD,
    QC_X86_INS_VMOVLPS,
    QC_X86_INS_VMOVMSKPD,
    QC_X86_INS_VMOVMSKPS,
    QC_X86_INS_VMOVNTDQA,
    QC_X86_INS_VMOVNTDQ,
    QC_X86_INS_VMOVNTPD,
    QC_X86_INS_VMOVNTPS,
    QC_X86_INS_VMOVSD,
    QC_X86_INS_VMOVSHDUP,
    QC_X86_INS_VMOVSLDUP,
    QC_X86_INS_VMOVSS,
    QC_X86_INS_VMOVUPD,
    QC_X86_INS_VMOVUPS,
    QC_X86_INS_VMPSADBW,
    QC_X86_INS_VMPTRLD,
    QC_X86_INS_VMPTRST,
    QC_X86_INS_VMREAD,
    QC_X86_INS_VMRESUME,
    QC_X86_INS_VMRUN,
    QC_X86_INS_VMSAVE,
    QC_X86_INS_VMULPD,
    QC_X86_INS_VMULPS,
    QC_X86_INS_VMULSD,
    QC_X86_INS_VMULSS,
    QC_X86_INS_VMWRITE,
    QC_X86_INS_VMXOFF,
    QC_X86_INS_VMXON,
    QC_X86_INS_VPABSB,
    QC_X86_INS_VPABSD,
    QC_X86_INS_VPABSQ,
    QC_X86_INS_VPABSW,
    QC_X86_INS_VPACKSSDW,
    QC_X86_INS_VPACKSSWB,
    QC_X86_INS_VPACKUSDW,
    QC_X86_INS_VPACKUSWB,
    QC_X86_INS_VPADDB,
    QC_X86_INS_VPADDD,
    QC_X86_INS_VPADDQ,
    QC_X86_INS_VPADDSB,
    QC_X86_INS_VPADDSW,
    QC_X86_INS_VPADDUSB,
    QC_X86_INS_VPADDUSW,
    QC_X86_INS_VPADDW,
    QC_X86_INS_VPALIGNR,
    QC_X86_INS_VPANDD,
    QC_X86_INS_VPANDND,
    QC_X86_INS_VPANDNQ,
    QC_X86_INS_VPANDN,
    QC_X86_INS_VPANDQ,
    QC_X86_INS_VPAND,
    QC_X86_INS_VPAVGB,
    QC_X86_INS_VPAVGW,
    QC_X86_INS_VPBLENDD,
    QC_X86_INS_VPBLENDMB,
    QC_X86_INS_VPBLENDMD,
    QC_X86_INS_VPBLENDMQ,
    QC_X86_INS_VPBLENDMW,
    QC_X86_INS_VPBLENDVB,
    QC_X86_INS_VPBLENDW,
    QC_X86_INS_VPBROADCASTB,
    QC_X86_INS_VPBROADCASTD,
    QC_X86_INS_VPBROADCASTMB2Q,
    QC_X86_INS_VPBROADCASTMW2D,
    QC_X86_INS_VPBROADCASTQ,
    QC_X86_INS_VPBROADCASTW,
    QC_X86_INS_VPCLMULQDQ,
    QC_X86_INS_VPCMOV,
    QC_X86_INS_VPCMPB,
    QC_X86_INS_VPCMPD,
    QC_X86_INS_VPCMPEQB,
    QC_X86_INS_VPCMPEQD,
    QC_X86_INS_VPCMPEQQ,
    QC_X86_INS_VPCMPEQW,
    QC_X86_INS_VPCMPESTRI,
    QC_X86_INS_VPCMPESTRM,
    QC_X86_INS_VPCMPGTB,
    QC_X86_INS_VPCMPGTD,
    QC_X86_INS_VPCMPGTQ,
    QC_X86_INS_VPCMPGTW,
    QC_X86_INS_VPCMPISTRI,
    QC_X86_INS_VPCMPISTRM,
    QC_X86_INS_VPCMPQ,
    QC_X86_INS_VPCMPUB,
    QC_X86_INS_VPCMPUD,
    QC_X86_INS_VPCMPUQ,
    QC_X86_INS_VPCMPUW,
    QC_X86_INS_VPCMPW,
    QC_X86_INS_VPCOMB,
    QC_X86_INS_VPCOMD,
    QC_X86_INS_VPCOMPRESSD,
    QC_X86_INS_VPCOMPRESSQ,
    QC_X86_INS_VPCOMQ,
    QC_X86_INS_VPCOMUB,
    QC_X86_INS_VPCOMUD,
    QC_X86_INS_VPCOMUQ,
    QC_X86_INS_VPCOMUW,
    QC_X86_INS_VPCOMW,
    QC_X86_INS_VPCONFLICTD,
    QC_X86_INS_VPCONFLICTQ,
    QC_X86_INS_VPERM2F128,
    QC_X86_INS_VPERM2I128,
    QC_X86_INS_VPERMD,
    QC_X86_INS_VPERMI2D,
    QC_X86_INS_VPERMI2PD,
    QC_X86_INS_VPERMI2PS,
    QC_X86_INS_VPERMI2Q,
    QC_X86_INS_VPERMIL2PD,
    QC_X86_INS_VPERMIL2PS,
    QC_X86_INS_VPERMILPD,
    QC_X86_INS_VPERMILPS,
    QC_X86_INS_VPERMPD,
    QC_X86_INS_VPERMPS,
    QC_X86_INS_VPERMQ,
    QC_X86_INS_VPERMT2D,
    QC_X86_INS_VPERMT2PD,
    QC_X86_INS_VPERMT2PS,
    QC_X86_INS_VPERMT2Q,
    QC_X86_INS_VPEXPANDD,
    QC_X86_INS_VPEXPANDQ,
    QC_X86_INS_VPEXTRB,
    QC_X86_INS_VPEXTRD,
    QC_X86_INS_VPEXTRQ,
    QC_X86_INS_VPEXTRW,
    QC_X86_INS_VPGATHERDD,
    QC_X86_INS_VPGATHERDQ,
    QC_X86_INS_VPGATHERQD,
    QC_X86_INS_VPGATHERQQ,
    QC_X86_INS_VPHADDBD,
    QC_X86_INS_VPHADDBQ,
    QC_X86_INS_VPHADDBW,
    QC_X86_INS_VPHADDDQ,
    QC_X86_INS_VPHADDD,
    QC_X86_INS_VPHADDSW,
    QC_X86_INS_VPHADDUBD,
    QC_X86_INS_VPHADDUBQ,
    QC_X86_INS_VPHADDUBW,
    QC_X86_INS_VPHADDUDQ,
    QC_X86_INS_VPHADDUWD,
    QC_X86_INS_VPHADDUWQ,
    QC_X86_INS_VPHADDWD,
    QC_X86_INS_VPHADDWQ,
    QC_X86_INS_VPHADDW,
    QC_X86_INS_VPHMINPOSUW,
    QC_X86_INS_VPHSUBBW,
    QC_X86_INS_VPHSUBDQ,
    QC_X86_INS_VPHSUBD,
    QC_X86_INS_VPHSUBSW,
    QC_X86_INS_VPHSUBWD,
    QC_X86_INS_VPHSUBW,
    QC_X86_INS_VPINSRB,
    QC_X86_INS_VPINSRD,
    QC_X86_INS_VPINSRQ,
    QC_X86_INS_VPINSRW,
    QC_X86_INS_VPLZCNTD,
    QC_X86_INS_VPLZCNTQ,
    QC_X86_INS_VPMACSDD,
    QC_X86_INS_VPMACSDQH,
    QC_X86_INS_VPMACSDQL,
    QC_X86_INS_VPMACSSDD,
    QC_X86_INS_VPMACSSDQH,
    QC_X86_INS_VPMACSSDQL,
    QC_X86_INS_VPMACSSWD,
    QC_X86_INS_VPMACSSWW,
    QC_X86_INS_VPMACSWD,
    QC_X86_INS_VPMACSWW,
    QC_X86_INS_VPMADCSSWD,
    QC_X86_INS_VPMADCSWD,
    QC_X86_INS_VPMADDUBSW,
    QC_X86_INS_VPMADDWD,
    QC_X86_INS_VPMASKMOVD,
    QC_X86_INS_VPMASKMOVQ,
    QC_X86_INS_VPMAXSB,
    QC_X86_INS_VPMAXSD,
    QC_X86_INS_VPMAXSQ,
    QC_X86_INS_VPMAXSW,
    QC_X86_INS_VPMAXUB,
    QC_X86_INS_VPMAXUD,
    QC_X86_INS_VPMAXUQ,
    QC_X86_INS_VPMAXUW,
    QC_X86_INS_VPMINSB,
    QC_X86_INS_VPMINSD,
    QC_X86_INS_VPMINSQ,
    QC_X86_INS_VPMINSW,
    QC_X86_INS_VPMINUB,
    QC_X86_INS_VPMINUD,
    QC_X86_INS_VPMINUQ,
    QC_X86_INS_VPMINUW,
    QC_X86_INS_VPMOVDB,
    QC_X86_INS_VPMOVDW,
    QC_X86_INS_VPMOVM2B,
    QC_X86_INS_VPMOVM2D,
    QC_X86_INS_VPMOVM2Q,
    QC_X86_INS_VPMOVM2W,
    QC_X86_INS_VPMOVMSKB,
    QC_X86_INS_VPMOVQB,
    QC_X86_INS_VPMOVQD,
    QC_X86_INS_VPMOVQW,
    QC_X86_INS_VPMOVSDB,
    QC_X86_INS_VPMOVSDW,
    QC_X86_INS_VPMOVSQB,
    QC_X86_INS_VPMOVSQD,
    QC_X86_INS_VPMOVSQW,
    QC_X86_INS_VPMOVSXBD,
    QC_X86_INS_VPMOVSXBQ,
    QC_X86_INS_VPMOVSXBW,
    QC_X86_INS_VPMOVSXDQ,
    QC_X86_INS_VPMOVSXWD,
    QC_X86_INS_VPMOVSXWQ,
    QC_X86_INS_VPMOVUSDB,
    QC_X86_INS_VPMOVUSDW,
    QC_X86_INS_VPMOVUSQB,
    QC_X86_INS_VPMOVUSQD,
    QC_X86_INS_VPMOVUSQW,
    QC_X86_INS_VPMOVZXBD,
    QC_X86_INS_VPMOVZXBQ,
    QC_X86_INS_VPMOVZXBW,
    QC_X86_INS_VPMOVZXDQ,
    QC_X86_INS_VPMOVZXWD,
    QC_X86_INS_VPMOVZXWQ,
    QC_X86_INS_VPMULDQ,
    QC_X86_INS_VPMULHRSW,
    QC_X86_INS_VPMULHUW,
    QC_X86_INS_VPMULHW,
    QC_X86_INS_VPMULLD,
    QC_X86_INS_VPMULLQ,
    QC_X86_INS_VPMULLW,
    QC_X86_INS_VPMULUDQ,
    QC_X86_INS_VPORD,
    QC_X86_INS_VPORQ,
    QC_X86_INS_VPOR,
    QC_X86_INS_VPPERM,
    QC_X86_INS_VPROTB,
    QC_X86_INS_VPROTD,
    QC_X86_INS_VPROTQ,
    QC_X86_INS_VPROTW,
    QC_X86_INS_VPSADBW,
    QC_X86_INS_VPSCATTERDD,
    QC_X86_INS_VPSCATTERDQ,
    QC_X86_INS_VPSCATTERQD,
    QC_X86_INS_VPSCATTERQQ,
    QC_X86_INS_VPSHAB,
    QC_X86_INS_VPSHAD,
    QC_X86_INS_VPSHAQ,
    QC_X86_INS_VPSHAW,
    QC_X86_INS_VPSHLB,
    QC_X86_INS_VPSHLD,
    QC_X86_INS_VPSHLQ,
    QC_X86_INS_VPSHLW,
    QC_X86_INS_VPSHUFB,
    QC_X86_INS_VPSHUFD,
    QC_X86_INS_VPSHUFHW,
    QC_X86_INS_VPSHUFLW,
    QC_X86_INS_VPSIGNB,
    QC_X86_INS_VPSIGND,
    QC_X86_INS_VPSIGNW,
    QC_X86_INS_VPSLLDQ,
    QC_X86_INS_VPSLLD,
    QC_X86_INS_VPSLLQ,
    QC_X86_INS_VPSLLVD,
    QC_X86_INS_VPSLLVQ,
    QC_X86_INS_VPSLLW,
    QC_X86_INS_VPSRAD,
    QC_X86_INS_VPSRAQ,
    QC_X86_INS_VPSRAVD,
    QC_X86_INS_VPSRAVQ,
    QC_X86_INS_VPSRAW,
    QC_X86_INS_VPSRLDQ,
    QC_X86_INS_VPSRLD,
    QC_X86_INS_VPSRLQ,
    QC_X86_INS_VPSRLVD,
    QC_X86_INS_VPSRLVQ,
    QC_X86_INS_VPSRLW,
    QC_X86_INS_VPSUBB,
    QC_X86_INS_VPSUBD,
    QC_X86_INS_VPSUBQ,
    QC_X86_INS_VPSUBSB,
    QC_X86_INS_VPSUBSW,
    QC_X86_INS_VPSUBUSB,
    QC_X86_INS_VPSUBUSW,
    QC_X86_INS_VPSUBW,
    QC_X86_INS_VPTESTMD,
    QC_X86_INS_VPTESTMQ,
    QC_X86_INS_VPTESTNMD,
    QC_X86_INS_VPTESTNMQ,
    QC_X86_INS_VPTEST,
    QC_X86_INS_VPUNPCKHBW,
    QC_X86_INS_VPUNPCKHDQ,
    QC_X86_INS_VPUNPCKHQDQ,
    QC_X86_INS_VPUNPCKHWD,
    QC_X86_INS_VPUNPCKLBW,
    QC_X86_INS_VPUNPCKLDQ,
    QC_X86_INS_VPUNPCKLQDQ,
    QC_X86_INS_VPUNPCKLWD,
    QC_X86_INS_VPXORD,
    QC_X86_INS_VPXORQ,
    QC_X86_INS_VPXOR,
    QC_X86_INS_VRCP14PD,
    QC_X86_INS_VRCP14PS,
    QC_X86_INS_VRCP14SD,
    QC_X86_INS_VRCP14SS,
    QC_X86_INS_VRCP28PD,
    QC_X86_INS_VRCP28PS,
    QC_X86_INS_VRCP28SD,
    QC_X86_INS_VRCP28SS,
    QC_X86_INS_VRCPPS,
    QC_X86_INS_VRCPSS,
    QC_X86_INS_VRNDSCALEPD,
    QC_X86_INS_VRNDSCALEPS,
    QC_X86_INS_VRNDSCALESD,
    QC_X86_INS_VRNDSCALESS,
    QC_X86_INS_VROUNDPD,
    QC_X86_INS_VROUNDPS,
    QC_X86_INS_VROUNDSD,
    QC_X86_INS_VROUNDSS,
    QC_X86_INS_VRSQRT14PD,
    QC_X86_INS_VRSQRT14PS,
    QC_X86_INS_VRSQRT14SD,
    QC_X86_INS_VRSQRT14SS,
    QC_X86_INS_VRSQRT28PD,
    QC_X86_INS_VRSQRT28PS,
    QC_X86_INS_VRSQRT28SD,
    QC_X86_INS_VRSQRT28SS,
    QC_X86_INS_VRSQRTPS,
    QC_X86_INS_VRSQRTSS,
    QC_X86_INS_VSCATTERDPD,
    QC_X86_INS_VSCATTERDPS,
    QC_X86_INS_VSCATTERPF0DPD,
    QC_X86_INS_VSCATTERPF0DPS,
    QC_X86_INS_VSCATTERPF0QPD,
    QC_X86_INS_VSCATTERPF0QPS,
    QC_X86_INS_VSCATTERPF1DPD,
    QC_X86_INS_VSCATTERPF1DPS,
    QC_X86_INS_VSCATTERPF1QPD,
    QC_X86_INS_VSCATTERPF1QPS,
    QC_X86_INS_VSCATTERQPD,
    QC_X86_INS_VSCATTERQPS,
    QC_X86_INS_VSHUFPD,
    QC_X86_INS_VSHUFPS,
    QC_X86_INS_VSQRTPD,
    QC_X86_INS_VSQRTPS,
    QC_X86_INS_VSQRTSD,
    QC_X86_INS_VSQRTSS,
    QC_X86_INS_VSTMXCSR,
    QC_X86_INS_VSUBPD,
    QC_X86_INS_VSUBPS,
    QC_X86_INS_VSUBSD,
    QC_X86_INS_VSUBSS,
    QC_X86_INS_VTESTPD,
    QC_X86_INS_VTESTPS,
    QC_X86_INS_VUNPCKHPD,
    QC_X86_INS_VUNPCKHPS,
    QC_X86_INS_VUNPCKLPD,
    QC_X86_INS_VUNPCKLPS,
    QC_X86_INS_VZEROALL,
    QC_X86_INS_VZEROUPPER,
    QC_X86_INS_WAIT,
    QC_X86_INS_WBINVD,
    QC_X86_INS_WRFSBASE,
    QC_X86_INS_WRGSBASE,
    QC_X86_INS_WRMSR,
    QC_X86_INS_XABORT,
    QC_X86_INS_XACQUIRE,
    QC_X86_INS_XBEGIN,
    QC_X86_INS_XCHG,
    QC_X86_INS_XCRYPTCBC,
    QC_X86_INS_XCRYPTCFB,
    QC_X86_INS_XCRYPTCTR,
    QC_X86_INS_XCRYPTECB,
    QC_X86_INS_XCRYPTOFB,
    QC_X86_INS_XEND,
    QC_X86_INS_XGETBV,
    QC_X86_INS_XLATB,
    QC_X86_INS_XRELEASE,
    QC_X86_INS_XRSTOR,
    QC_X86_INS_XRSTOR64,
    QC_X86_INS_XRSTORS,
    QC_X86_INS_XRSTORS64,
    QC_X86_INS_XSAVE,
    QC_X86_INS_XSAVE64,
    QC_X86_INS_XSAVEC,
    QC_X86_INS_XSAVEC64,
    QC_X86_INS_XSAVEOPT,
    QC_X86_INS_XSAVEOPT64,
    QC_X86_INS_XSAVES,
    QC_X86_INS_XSAVES64,
    QC_X86_INS_XSETBV,
    QC_X86_INS_XSHA1,
    QC_X86_INS_XSHA256,
    QC_X86_INS_XSTORE,
    QC_X86_INS_XTEST,
    QC_X86_INS_FDISI8087_NOP,
    QC_X86_INS_FENI8087_NOP,

    QC_X86_INS_ENDING, // mark the end of the list of insn
} qc_x86_insn;

#ifdef __cplusplus
}
#endif

#endif
